/*
 * KernelMAP Visualization program
 *
 * Author: j00ru//vx
 * Date:   2010-01-02
 * E-mail: j00ru.vx@gmail.com
 * WWW:    http://j00ru.vexillium.org/
 *
 * LICENSE
 * Permission is hereby granted to use, copy, modify, and distribute this source code, 
 * or portions hereof, documentation and executables, for any purpose, without fee, 
 * subject to the following restrictions:
 * 
 * 1. The origin of this source code must not be misrepresented.
 *
 * 2. Altered versions must be plainly marked as such and must not be misrepresented 
 *    as being the original source.
 *
 * 3. This Copyright notice may not be removed or altered from any source or altered 
 *    source distribution.
 * 
 * This software is provided "AS IS". The author does not guarantee that this program works 
 * or is bug-free. The author takes no responsibility for any damage caused by this program.
 * Use at your own risk.
 */
#pragma once

#include <vector>
#include <windows.h>
#include <ddk\ntapi.h>
#include <ddk\ntddk.h>
#include <ddk\winddk.h>

#include "CLog.h"
#include "def.h"
using namespace std;


/* XXX
 */
extern CLog LogClass;

/* Available types of memory blocks
 */
enum AddressType
{
  UnknownType,
  ModuleImageBase,
  HandleAllocation,
  LockAllocation,
  GDT,
  LDT,
  IDT
};

/* A structure containing all the desired information about
 * any given kernel address retrieved by one of the modules.
 */
struct AddressDescriptor
{
  /* The type of the kernel address in consideration.
   * The list below can be easily extented with additional types, in the future.
   */
  AddressType Type;

  /* A simple union containing information about all types specified in the AddressType enum.
   */
  union
  {
    struct
    {
      /* The file path of the kernel module.
       */
      CHAR FileName[MAX_PATH];
    } KernelModule;

    /* Detailed information about the specific HANDLE
     */
    SYSTEM_HANDLE_TABLE_ENTRY_INFO KernelHandle;

    /* System Lock structure
     */
    SYSTEM_LOCK_INFORMATION        KernelLock;

    /* Number of the GDT/IDT owner processor
     */
    ULONG ProcessNumber;

  } AddressInfo;

  /* The KERNEL-MODE Address itself
   */
  LPVOID Address;

  /* Just size
   */
  ULONG Size;

  friend bool operator< (AddressDescriptor a, AddressDescriptor b)
  {
    return a.Address < b.Address;
  }
};



/* Base class for specific modules obtaining information about the
 * kernel mode addresses implemented in separate files.
 */
class CKernelAddrModule
{
  public:
    vector<AddressDescriptor> KM_Addresses;

    CKernelAddrModule()
    {
      this->KM_Addresses.clear();
    }

    ~CKernelAddrModule()
    {
    }

    virtual BOOL Run()
    {
      return TRUE;
    }

    virtual ULONG Size()
    {
      return this->KM_Addresses.size();
    }

    virtual BOOL GetAddressInfo(ULONG Index, AddressDescriptor* OutputAddress)
    {
      if(Index >= this->KM_Addresses.size())
        return FALSE;

      memcpy(OutputAddress,&this->KM_Addresses[Index],sizeof(AddressDescriptor));
      return TRUE;
    }

    virtual VOID PrintConsoleInfo()
    {
    }
};

