/*
 * KernelMAP Visualization program
 *
 * Author: j00ru//vx
 * Date:   2010-01-02
 * E-mail: j00ru.vx@gmail.com
 * WWW:    http://j00ru.vexillium.org/
 *
 * LICENSE
 * Permission is hereby granted to use, copy, modify, and distribute this source code, 
 * or portions hereof, documentation and executables, for any purpose, without fee, 
 * subject to the following restrictions:
 * 
 * 1. The origin of this source code must not be misrepresented.
 *
 * 2. Altered versions must be plainly marked as such and must not be misrepresented 
 *    as being the original source.
 *
 * 3. This Copyright notice may not be removed or altered from any source or altered 
 *    source distribution.
 * 
 * This software is provided "AS IS". The author does not guarantee that this program works 
 * or is bug-free. The author takes no responsibility for any damage caused by this program.
 * Use at your own risk.
 */
#pragma once

#include <cstdio>
#include <cstdlib>
#include <string>
#include <windows.h>
using namespace std;

/* Just a logging/console handling class
 */
class CLog
{
  private:
    BOOL Debugged;
    WORD OriginalAttributes;

  public:
    CLog()
    {
      /* Debugging signals disabled by default
       */
      this->Debugged = FALSE;

      /* Allocate the text-mode console
       */
      AllocConsole();

      /* Retrieve the original text attributes
       */
      CONSOLE_SCREEN_BUFFER_INFO ScreenBufferInfo;
      GetConsoleScreenBufferInfo(GetStdHandle(STD_OUTPUT_HANDLE),&ScreenBufferInfo);
      this->OriginalAttributes = ScreenBufferInfo.wAttributes;
    }

    ~CLog()
    {
      /* Free the console
       */
      FreeConsole();
    }

    VOID SetDebugged(BOOL Value)
    {
      this->Debugged = Value;
    }


    BOOL Debug(const CHAR* Format, ...)
    {
      va_list Args;

      if(this->Debugged == FALSE)
        return FALSE;
      
      fprintf(stderr,"[DEBUG] ");
      va_start(Args,Format);
      vfprintf(stderr,Format,Args);
      va_end(Args);
      return TRUE;
    }

    VOID SetXY(UINT X, UINT Y)
    {
      COORD XY = {X,Y};      
      SetConsoleCursorPosition(GetStdHandle(STD_OUTPUT_HANDLE),XY);      
    }

    BOOL Print(WORD Attributes, const CHAR* Format, ...)
    {
      static CHAR Buffer[1024];
      DWORD NumberOfBytesWritten;
      va_list Args;

      if(Attributes != 0)
        SetConsoleTextAttribute(GetStdHandle(STD_OUTPUT_HANDLE),Attributes);        

      va_start(Args,Format);

      vsnprintf(Buffer,sizeof(Buffer),Format,Args);
      WriteFile(GetStdHandle(STD_OUTPUT_HANDLE),Buffer,strlen(Buffer),&NumberOfBytesWritten,NULL);
      SetConsoleTextAttribute(GetStdHandle(STD_OUTPUT_HANDLE),this->OriginalAttributes);

      va_end(Args);
      return TRUE;      
    }
};

