/*
 * KernelMAP Visualization program
 *
 * Author: j00ru//vx
 * Date:   2010-01-02
 * E-mail: j00ru.vx@gmail.com
 * WWW:    http://j00ru.vexillium.org/
 *
 * LICENSE
 * Permission is hereby granted to use, copy, modify, and distribute this source code, 
 * or portions hereof, documentation and executables, for any purpose, without fee, 
 * subject to the following restrictions:
 * 
 * 1. The origin of this source code must not be misrepresented.
 *
 * 2. Altered versions must be plainly marked as such and must not be misrepresented 
 *    as being the original source.
 *
 * 3. This Copyright notice may not be removed or altered from any source or altered 
 *    source distribution.
 * 
 * This software is provided "AS IS". The author does not guarantee that this program works 
 * or is bug-free. The author takes no responsibility for any damage caused by this program.
 * Use at your own risk.
 */
#pragma once

#include <cstdio>
#include <cstdlib>
#include <string>
#include <windows.h>
#include <png.h>

#include "CGlobalInformation.h"
using namespace std;

class CPNG
{
  private:
    ULONG Width, Height;
    FILE* File;

    png_bytep* RowPointers;
    png_structp png_ptr;
    png_infop info_ptr;    

  public:
    CPNG()
    {
      this->Width  = 0;
      this->Height = 0;
      
      this->File        = NULL;
      this->RowPointers = NULL;
      
      memset(&this->png_ptr,0,sizeof(this->png_ptr));
      memset(&this->info_ptr,0,sizeof(this->info_ptr));
    }

    ~CPNG()
    {
      for( ULONG i=0;i<Height;i++ )
      {
        free(this->RowPointers[i]);
      }

      if(this->File)
        fclose(this->File);
    }

    BOOL OpenFile(const CHAR* FileName, ULONG Width, ULONG Height)
    {
      LogClass.Debug("CPNG::OpenFile(FileName = %s, Width = %d, Height = %d) called.\n",
          FileName,Width,Height);

      /* Open the specified file
       */
      this->File = fopen(FileName,"wb");
      if(this->File == NULL)
        return FALSE;

      /* ----- Some libpng code :o
       *       Better not change it, can be dangerous :o
       */
      this->png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
      if(!this->png_ptr)
      {
        fclose(this->File);
        this->File = NULL;
        return FALSE;
      }

      this->info_ptr = png_create_info_struct(png_ptr);
      if(!this->info_ptr)
      {
        fclose(this->File);
        this->File = NULL;
        return FALSE;
      }

      /* Set some private variables
       */
      this->Width  = Width;
      this->Height = Height;

      /* Allocate memory for the raw image data
       */
      this->RowPointers = (png_bytep*)malloc(sizeof(png_bytep)*Height);
      for( ULONG i=0;i<Height;i++ )
      {
        this->RowPointers[i] = (png_bytep)malloc(Width*3);
        memset(this->RowPointers[i],0x00,Width*3);
      }

      return TRUE;
    }

    BOOL SetPixel(ULONG Row, ULONG Column, BYTE r, BYTE g, BYTE b)
    {
      /* Some boundary check
       */
      if(Row>=this->Height || Column>=this->Width)
        return FALSE;

      /* Set the r,g,b colors
       */
      RowPointers[Row][Column*3+0] = r;
      RowPointers[Row][Column*3+1] = g;
      RowPointers[Row][Column*3+2] = b;

      return TRUE;
    }

    BOOL Save()
    {
      if(this->File == NULL)
        return FALSE;

      /* Another piece of libpng code :(
       */
      png_init_io(this->png_ptr, this->File);
      png_set_IHDR(this->png_ptr, this->info_ptr, this->Width, this->Height,
                   8, PNG_COLOR_TYPE_RGB, PNG_INTERLACE_NONE,
                   PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);
      png_write_info(this->png_ptr, this->info_ptr);
      png_write_image(this->png_ptr, this->RowPointers);
      png_write_end(this->png_ptr, NULL);

      /* Close the file handle, eventually
       */
      fclose(this->File);
      return TRUE;
    }
};
